//
//  FirmwareImageTableViewController.m
//  Bluzone
//
//  Created by Jason Clary on 7/28/16.
//  Copyright © 2016 Bluvision. All rights reserved.
//

#import "FirmwareImageTableViewController.h"
#import "FirmwareStorage.h"
#import "FirmwareUpdater.h"

#import <Bluvision/Bluvision.h>

@interface FirmwareImageTableViewController () <FirmwareUpdaterDelegate>

@property (nonatomic, strong) NSArray *firmwareImages;
@property (nonatomic, strong) BLUFirmwareImage *loaderImage;
@property (nonatomic, strong) FirmwareUpdater *firmwareUpdater;

@end

@implementation FirmwareImageTableViewController

- (void)viewDidLoad {
    [super viewDidLoad];

    self.firmwareUpdater = [[FirmwareUpdater alloc] init];

    NSData *firmwareData = [NSData dataWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"firmware-cardv3" ofType:@"sfw"]];
    NSArray *images = [BLUFirmwareImage firmwareImagesForData:firmwareData configurableBeacon:self.beacon];

    NSMutableArray *nonLoaderImages = [NSMutableArray new];

    for (BLUFirmwareImage *image in images) {
        if ([image.firmwareName localizedCaseInsensitiveContainsString:@"loader"] || [image.firmwareDescription localizedCaseInsensitiveContainsString:@"loader"]) {
            self.loaderImage = image;
            continue;
        }

        [nonLoaderImages addObject:image];
    }

    self.firmwareImages = [NSArray arrayWithArray:nonLoaderImages];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.firmwareImages.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"Cell" forIndexPath:indexPath];

    BLUFirmwareImage *image = self.firmwareImages[indexPath.row];

    cell.textLabel.text = image.firmwareDescription;
    cell.detailTextLabel.text = image.firmwareName;

    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [self showFirmwareUpdateConfirmationAlert];
}

- (void)showFirmwareUpdateConfirmationAlert {
    self.firmwareUpdater.configurableBeacon = self.beacon;
    self.firmwareUpdater.stage1FirmwareImage = self.loaderImage;
    self.firmwareUpdater.stage2FirmwareImage = self.firmwareImages[self.tableView.indexPathForSelectedRow.row];
    self.firmwareUpdater.delegate = self;


    UIAlertController *alert =  [UIAlertController alertControllerWithTitle:@"Firmware Update" message:@"Would you like to begin updating the firmware of this beacon?" preferredStyle:UIAlertControllerStyleAlert];
    [alert addAction:[UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:nil]];
    [alert addAction:[UIAlertAction actionWithTitle:@"Install Firmware" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        [self.firmwareUpdater beginFirmwareUpdate];
    }]];

    [self presentViewController:alert animated:YES completion:nil];
}


- (void)firmwareUpdater:(FirmwareUpdater *)firmwareUpdater didFinishFirmwareUpdateWithError:(NSError *)error {
    [self.tableView deselectRowAtIndexPath:self.tableView.indexPathForSelectedRow animated:YES];

    if (error) {
        NSString *underlyingErrorMessage = [error.userInfo[NSUnderlyingErrorKey] localizedDescription] ?: nil;
        underlyingErrorMessage = underlyingErrorMessage ? [NSString stringWithFormat:@" - %@", underlyingErrorMessage] : @"";

        [self showAlertWithMessage:[NSString stringWithFormat:@"%@%@",error.localizedDescription, underlyingErrorMessage]];
    }
    else {
        [self showAlertWithMessage:@"Firmware Update Completed"];
        [self.navigationController popViewControllerAnimated:YES];
    }
}

- (void)showAlertWithMessage:(NSString *)message {
    UIAlertController *alert = [UIAlertController alertControllerWithTitle:@"Mesage" message:message preferredStyle:UIAlertControllerStyleAlert];

    [self presentViewController:alert animated:YES completion:nil];
}

@end
